package Source::Shared::Resource::Verbose;

use strict;
use warnings;

use Source::Shared;
use Source::Shared::Resource;
use Source::Shared::ResourceContainer;
our @ISA=qw/Source::Shared::Resource/;

unshift @Source::Shared::ResourceContainer::RESOURCE_CLASSES, __PACKAGE__;

our $VERSION=0.001;
our $verbose=0;

our @LONGOPT=(
    'v|verbose+'  => \$Source::Shared::verbose,
    'q|quiet'   => sub { $Source::Shared::verbose = 0 },
);

sub print_longopt {
    print "Common verbosity options:
	-v, --verbose (can be specified multiple times)
	-q, --quiet
";
}

sub process_options {
    my $newlevel;
    if ($Source::Shared::verbose==1) {
	# default; do nothing
	return;
    } elsif ($Source::Shared::verbose==0) {
	# global quiet
	$newlevel=0;
    } elsif ($Source::Shared::verbose>1) {
	# global verbose
	$newlevel=$Source::Shared::verbose - 1;
    }
    $verbose = $newlevel if $verbose < $newlevel;
    no strict "refs";
    print STDERR 'DEBUG: SET Verbose: containers: ',join(',',@Source::Shared::VERBOSE_CLASSES),"\n" if $verbose>3;

    foreach my $vcontainer (@Source::Shared::VERBOSE_CLASSES) {
	foreach my $vclass ($vcontainer->get_cli_classes) {
	    my $verbose_stringref="${vclass}::verbose";
	    my $verbose_ref=*{$verbose_stringref}{SCALAR};
	    unless ($verbose_ref) {
		print STDERR "DEBUG: SET Verbose: skipped $vclass -- no \$verbose found\n" if $verbose>3;
		next;
	    } else {
		if (not $$verbose_ref or $$verbose_ref < $newlevel) {
		    print STDERR "DEBUG: SET Verbose: set ${vclass}::verbose to $newlevel\n" if $verbose>3;
		    $$verbose_ref=$newlevel;
		} else {
		    print STDERR "DEBUG: SET Verbose: skip ${vclass}::verbose -- already $$verbose_ref\n" if $verbose>3;
		}
	    }
	}
    }
}

1;
